
import { createRoot } from 'react-dom/client'
import { lazy, Suspense, useEffect } from 'react'
import './index.css'
import { supabase, getAuthClient, preconnectSupabase } from '@/integrations/supabase/client'
import { getCacheControlHeader, addPreloadHints } from '@/utils/cacheMiddleware'
import { AuthProvider } from '@/contexts/auth/AuthProvider'
import { APP_VERSION, checkForAppUpdate, forceAppRefresh } from '@/utils/cacheUtils'

// Initialize preloading as early as possible
preconnectSupabase();

// Define the prefetchRoute function
const prefetchRoute = (path: string) => {
  const link = document.createElement('link');
  link.rel = 'prefetch';
  link.href = path;
  
  // Add appropriate cache headers based on the asset type
  const cacheControl = getCacheControlHeader(path);
  if (cacheControl) {
    link.setAttribute('data-cache-control', cacheControl);
  }
  
  document.head.appendChild(link);
};

// Import App directly without lazy loading to avoid loading state
import App from './App.tsx';

// Initialize app with optimized loading
const initApp = async () => {
  // Get the root element
  const root = document.getElementById("root")!;
  
  // Add version to prevent Safari caching issues
  if (typeof window !== 'undefined') {
    // Add version meta tag to force cache refresh
    const meta = document.createElement('meta');
    meta.name = 'app-version';
    meta.content = APP_VERSION;
    document.head.appendChild(meta);
    
    // Check if app needs to be refreshed due to version change
    if (checkForAppUpdate()) {
      console.log('New application version detected, refreshing...');
      forceAppRefresh();
      return; // Stop initialization as page will reload
    }
    
    // Add cache control headers
    const metaCache = document.createElement('meta');
    metaCache.httpEquiv = 'Cache-Control';
    metaCache.content = 'no-cache, no-store, must-revalidate';
    document.head.appendChild(metaCache);
    
    const metaPragma = document.createElement('meta');
    metaPragma.httpEquiv = 'Pragma';
    metaPragma.content = 'no-cache';
    document.head.appendChild(metaPragma);
    
    const metaExpires = document.createElement('meta');
    metaExpires.httpEquiv = 'Expires';
    metaExpires.content = '0';
    document.head.appendChild(metaExpires);
    
    // Preload critical resources that will be needed immediately
    addPreloadHints(document, [
      { path: '/favicon.svg', as: 'image' },
      { path: '/src/components/Navbar.tsx', as: 'script' },
      { path: '/src/components/Footer.tsx', as: 'script' }
    ]);
  }
  
  // Start auth check immediately 
  const authPromise = preloadAuthState();
  
  // Now that we know auth status, preload appropriate routes
  const { data } = await authPromise;
  
  if (data?.session) {
    console.log('User is authenticated, prefetching dashboard');
    prefetchRoute('/dashboard');
  } else {
    console.log('User is not authenticated, prefetching auth page');
    prefetchRoute('/auth');
    prefetchRoute('/src/pages/Auth.tsx');
  }
  
  // Initialize React with AuthProvider wrapping the App
  createRoot(root).render(
    <AuthProvider>
      <App />
    </AuthProvider>
  );
};

// Lightweight auth state preloader
const preloadAuthState = async () => {
  try {
    // Start with inline session check for maximum speed
    const cachedAuth = localStorage.getItem('supabase-auth');
    if (cachedAuth) {
      try {
        const session = JSON.parse(cachedAuth);
        // Super quick validation
        if (session && session.access_token && new Date(session.expires_at) > new Date()) {
          console.log('Using cached auth session');
          // Use this immediately without waiting for validation
          return { data: { session } };
        }
      } catch (e) {
        // Invalid cache, will proceed with normal auth check
      }
    }
    
    // Use lightweight auth client to avoid full SDK initialization
    const authClient = getAuthClient();
    // Regular auth check
    return await authClient.auth.getSession();
  } catch (err) {
    console.error('Failed to pre-fetch auth session:', err);
    return { data: { session: null } };
  }
};

// Start the optimized initialization
initApp().catch(e => {
  console.error('Failed to initialize app:', e);
  // Fallback to direct rendering if optimized init fails
  createRoot(document.getElementById("root")!).render(
    <AuthProvider>
      <App />
    </AuthProvider>
  );
});
